import { Bucket } from '@aws-cdk/aws-s3';
import { ReceiptRuleSet } from '@aws-cdk/aws-ses';
import { Topic } from '@aws-cdk/aws-sns';
import { Construct } from '@aws-cdk/core';
import { NotificationType } from '@seeebiii/ses-verify-identities';
import { EmailMapping } from './email-forwarding-rule';
/**
 * @stability stable
 */
export interface EmailForwardingProps {
    /**
     * The domain name for which you want to receive emails using SES, e.g. `example.org`.
     *
     * @stability stable
     */
    readonly domainName: string;
    /**
     * Optional: true if you want to verify the domain identity in SES, false otherwise.
     *
     * @default false
     * @stability stable
     */
    readonly verifyDomain?: boolean;
    /**
     * A prefix that is used as the sender address of the forwarded mail, e.g. `noreply`.
     *
     * @stability stable
     */
    readonly fromPrefix: string;
    /**
     * A list of email mappings to define the receive email address and target email addresses to which the emails are forwarded to.
     *
     * @see EmailMapping
     * @stability stable
     */
    readonly emailMappings: EmailMapping[];
    /**
     * Optional: true if you want to initiate the verification of your target email addresses, false otherwise.
     *
     * If `true`, a verification email is sent out to all target email addresses. Then, the owner of an email address needs to verify it by clicking the link in the verification email.
     * Please note in case you don't verify your sender domain, it's required to verify your target email addresses in order to send mails to them.
     *
     * @default false
     * @stability stable
     */
    readonly verifyTargetEmailAddresses?: boolean;
    /**
     * Optional: an S3 bucket to store the received emails.
     *
     * If none is provided, a new one will be created.
     *
     * @default A new bucket.
     * @stability stable
     */
    readonly bucket?: Bucket;
    /**
     * Optional: a prefix for the email files that are stored on the S3 bucket.
     *
     * @default inbox/
     * @stability stable
     */
    readonly bucketPrefix?: string;
    /**
     * Optional: an SNS topic to receive notifications about sending events like bounces or complaints.
     *
     * The events are defined by `notificationTypes` using {@link NotificationType}. If no topic is defined, a new one will be created.
     *
     * @default A new SNS topic.
     * @stability stable
     */
    readonly notificationTopic?: Topic;
    /**
     * Optional: a list of {@link NotificationType}s to define which sending events should be subscribed.
     *
     * @default ['Bounce', 'Complaint']
     * @stability stable
     */
    readonly notificationTypes?: NotificationType[];
}
/**
 * @stability stable
 */
export interface EmailForwardingRuleSetProps {
    /**
     * Optional: an existing SES receipt rule set.
     *
     * If none is provided, a new one will be created using the name provided with `ruleSetName` or a default one.
     *
     * @stability stable
     */
    readonly ruleSet?: ReceiptRuleSet;
    /**
     * Optional: provide a name for the receipt rule set that this construct creates if you don't provide one.
     *
     * @default custom-rule-set
     * @stability stable
     */
    readonly ruleSetName?: string;
    /**
     * Optional: whether to enable the rule set or not.
     *
     * @default true
     * @stability stable
     */
    readonly enableRuleSet?: boolean;
    /**
     * A list of mapping options to define how emails should be forwarded.
     *
     * @stability stable
     */
    readonly emailForwardingProps: EmailForwardingProps[];
}
/**
 * A construct for AWS SES to forward all emails of certain domains and email addresses to a list of target email addresses.
 *
 * It also verifies (or at least initiates verification of) the related domains and email addresses in SES.
 *
 * The construct can be helpful if you don't want to host your own SMTP server but still want to receive emails to your existing email inbox.
 * One use case is if you're just building some sort of landing page and want to quickly setup email receiving for your domain without yet another separate email inbox.
 *
 * This construct can...
 * - create a new receipt rule set (or use an existing one),
 * - attach a list of rules to forward incoming emails to other target email addresses,
 * - verify a given domain in SES (automatically if domain is managed by Route53, otherwise it'll just initiate the verification),
 * - initiate verification for all target email addresses that are provided for receiving the forwarded emails.
 *
 * @stability stable
 */
export declare class EmailForwardingRuleSet extends Construct {
    /**
     * @stability stable
     */
    ruleSet: ReceiptRuleSet;
    /**
     * @stability stable
     */
    constructor(parent: Construct, name: string, props: EmailForwardingRuleSetProps);
    private createRuleSetOrUseExisting;
    private setupEmailForwardingMappings;
    private verifyDomain;
    private verifyTargetEmailAddresses;
    private enableRuleSet;
}
