import { Bucket } from '@aws-cdk/aws-s3';
import { ReceiptRuleSet } from '@aws-cdk/aws-ses';
import { Construct } from '@aws-cdk/core';
/**
 * @stability stable
 */
export interface EmailMapping {
    /**
     * You can define a string that is matching an email address, e.g. `hello@example.org`.
     *
     * If this property is defined, the `receivePrefix` will be ignored. You must define either this property or `receivePrefix`, otherwise no emails will be forwarded.
     *
     * @stability stable
     */
    readonly receiveEmail?: string;
    /**
     * A short way to match a specific email addresses by only providing a prefix, e.g. `hello`. The prefix will be combined with the given domain name from {@link IEmailForwardingRuleProps}. If an email was sent to this specific email address, all emails matching this receiver will be forwarded to all email addresses defined in `targetEmails`.
     *
     * If `receiveEmail` property is defined as well, then `receiveEmail` is preferred. Hence, only define one of them.
     *
     * @stability stable
     */
    readonly receivePrefix?: string;
    /**
     * A list of target email addresses that should receive the forwarded emails for the given email addresses matched by either `receiveEmail` or `receivePrefix`.
     *
     * Make sure that you only specify email addresses that are verified by SES. Otherwise SES won't send them out.
     *
     * Example: `['foobar@gmail.com', 'foo+bar@gmail.com', 'whatever@example.org']`
     *
     * @stability stable
     */
    readonly targetEmails: string[];
}
/**
 * @stability stable
 */
export interface EmailForwardingRuleProps {
    /**
     * An id for the rule.
     *
     * This will mainly be used to provide a name to the underlying rule but may also be used as a prefix for other resources.
     *
     * @stability stable
     */
    readonly id: string;
    /**
     * The rule set this rule belongs to.
     *
     * @stability stable
     */
    readonly ruleSet: ReceiptRuleSet;
    /**
     * The domain name of the email addresses, e.g. 'example.org'. It is used to connect the `fromPrefix` and `receivePrefix` properties with a proper domain.
     *
     * @stability stable
     */
    readonly domainName: string;
    /**
     * A prefix that is used as the sender address of the forwarded mail, e.g. `noreply`.
     *
     * @stability stable
     */
    readonly fromPrefix: string;
    /**
     * An email mapping similar to what the NPM library `aws-lambda-ses-forwarder` expects.
     *
     * @see EmailMapping
     * @stability stable
     */
    readonly emailMapping: EmailMapping[];
    /**
     * A bucket to store the email files to.
     *
     * If no bucket is provided, a new one will be created using a managed KMS key to encrypt the bucket.
     *
     * @default A new bucket will be created.
     * @stability stable
     */
    readonly bucket?: Bucket;
    /**
     * A prefix for the email files that are saved to the bucket.
     *
     * @default inbox/
     * @stability stable
     */
    readonly bucketPrefix?: string;
}
/**
 * A construct to define an email forwarding rule that can either be used together with {@link EmailForwardingRuleSet} or as a standalone rule.
 *
 * It creates two rule actions:
 * - One S3 action to save all incoming mails to an S3 bucket.
 * - One Lambda action to forward all incoming mails to a list of configured emails.
 *
 * The Lambda function is using the NPM package `aws-lambda-ses-forwarder` to forward the mails.
 *
 * @stability stable
 */
export declare class EmailForwardingRule extends Construct {
    /**
     * @stability stable
     */
    constructor(parent: Construct, name: string, props: EmailForwardingRuleProps);
    private createS3Action;
    private getBucketPrefixOrDefault;
    private createBucketOrUseExisting;
    private createReceiptRule;
    private mapForwardMappingToMap;
    private createLambdaForwarderAction;
    private createLambdaForwarderFunction;
}
